<?PHP
/*
	Prototype CRON/Scheduled script for the SNMP device polling.
	Adam Boutcher
	bd84dz@student.sunderland.ac.uk / adam.boutcher@gmail.com
	Requires: PHP5, PHP-SNMP, PHP-SQLite (SQLite3 PDO)
*/
// Put a PHP4 check here eventually as this is PHP5 and above
date_default_timezone_set('Europe/London');

// This page will be for the XML Cache generation
$PAGE_NAME = "../cache/printer_stats.xml";
$PAGE_OUTPUT = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
// Require The Printer Functions
	require("functions/printers/info.php");
	
// This is always FIXED
	$TONERS = array("black","cyan","magenta","yellow","imagetransfer","fuser","imagingunit");
	
// Databases
	$db_handle = new PDO("sqlite:databases/devices.database.sqlite3");
	$db_handle2 = new PDO("sqlite:databases/list.database.sqlite3");
	$db_handle3 = new PDO("sqlite:databases/alerts.database.sqlite3");
	
// Limit the number of requests to every 15mins
	$db_query = $db_handle2->query("SELECT val FROM system WHERE var = 'cache_time';");
	foreach($db_query as $row) {
		$_CONFIG['cache_time'] = $row['val'];
	}
	unset($db_query);
	
// Get Config Variables
	$db_query = $db_handle2->query("SELECT val FROM system WHERE var = 'whelp2';");
	foreach($db_query as $row) {
		$_CONFIG['helpdesk'] = $row['val'];
	}
	unset($db_query);
	$db_query = $db_handle2->query("SELECT val FROM system WHERE var = 'consumables_low';");
	foreach($db_query as $row) {
		$_CONFIG['low'] = $row['val'];
	}
	unset($db_query);
	$db_query = $db_handle2->query("SELECT val FROM system WHERE var = 'consumables_crit';");
	foreach($db_query as $row) {
		$_CONFIG['crit'] = $row['val'];
	}
	unset($db_query);
	
	// Does the cache Exist before we start asking for dates and stuff...
	// Timestamps are in POSIX for easier comparison checks.
	if(file_exists($PAGE_NAME)) {
		// Get the cache modifcation time and then add the time we expect it to be when it needs regenerating.
		$PAGE_AGE = filemtime($PAGE_NAME);
		$NEW_AGE = $PAGE_AGE+$_CONFIG['cache_time'];
	} else {
		// No cache = make one!
		$NEW_AGE = 0;
	}
	
	// Is the expected re-cache time less than the time now?
	if ($NEW_AGE <= time()) {
	
		// Find the Printer IDs
		$db_query = $db_handle->query("SELECT * FROM types WHERE type = 'laser printer';");
		foreach($db_query as $row) {
			$printer_id = $row['id'];
		}
		unset($db_query);
		
		// Find the Printers
		$db_query = $db_handle2->query("SELECT * FROM device_list WHERE type_id = '".$printer_id."';");
		foreach($db_query as $row) {
			$printers['id'][] = $row['id'];
			$printers['ip'][] = $row['ip'];
			$printers['ro_comm'][] = $row['ro_comm'];
		}
		unset($db_query);
		// Add the cache time, probably useful for something later.
		$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t<status cached=\"".date("g:i:sa - D jS F Y \(T\)")."\">";
	
		// Cycle through the printers
		$li = 0;
		do {
			$errors = 0;
			$email = 0;
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t<printer>";
			
			// is the printer reachable?
		if (ping($printers['ip'][$li]) == "up") {
			// get the info we need to then create something useful
			$printer_description = get_device_desc($printers['ip'][$li], $printers['ro_comm'][$li]);
			$printer_trans = get_device_info($printer_description);
			$status = get_printer_status($printers['ip'][$li], $printers['ro_comm'][$li],$printer_trans['id']);
			$printer_id = check_deivce_id($printers['ip'][$li], $printer_trans['uniq']);
			
			// output the ifno into the useful we mentioned earlier
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<description>".$printer_trans['desc']."</description>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<status>".$status."</status>";
			
// Status Error (4)
			// Does the status report an error?
			if (get_printer_status_test($printer_trans['id'], strtolower($status))>0) {
				// add the alert to the database ready for sending.
				make_alert(4, $printer_trans['uniq'], $status);
				make_alert_hd(4, $printer_trans['uniq'], $status);
				$errors++;	// Error Count (For XML)
				$email++;	// Error Count (for Alerting)
			}
			
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<address>".$printers['ip'][$li]."</address>";
			
			// Cycle through the toners
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<consumables value=\"Percent Remaining\">";
			$ti = 0;
			do {
				// Suppresed as if they don't exists you get errors
				$printer_toner_percent = @get_printer_toner_levels($printers['ip'][$li], $printers['ro_comm'][$li], $TONERS[$ti], $printer_trans['id']);
				$printer_toner_name = @get_printer_toner_names($printers['ip'][$li], $printers['ro_comm'][$li], $TONERS[$ti], $printer_trans['id']);
				// If the info is empty then the toner doesn't exist
				if (!empty($printer_toner_percent)) {
					$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t\t<".$TONERS[$ti]." name=\"".rtrim($printer_toner_name)."\">".round($printer_toner_percent)."</".$TONERS[$ti].">";
					
					// Make the toner name/part suitable for the alert database (The final variable is a rudamentary array, see the function for info)
					$printer_toner_name_db = str_replace(",", "", $printer_toner_name);
					if ($printer_toner_percent <= 0) {
// No Toner Error (3)
						make_alert(3, $printer_trans['uniq'], $printer_toner_name_db);
						make_alert_hd(3, $printer_trans['uniq'], $printer_toner_name_db);
						$errors++;
						$email++;
					} elseif (($printer_toner_percent <= $_CONFIG['crit'])&&(!empty($_CONFIG['crit']))) {
// Criticaly Low Toner Error (2)
						make_alert(2, $printer_trans['uniq'], $printer_toner_name_db);
						make_alert_hd(2, $printer_trans['uniq'], $printer_toner_name_db);
						$errors++;
						$email++;
					} elseif (($printer_toner_percent <= $_CONFIG['low'])&&(!empty($_CONFIG['low']))) {
// Low Toner Error (1), notify but don't colour
						make_alert(1, $printer_trans['uniq'], $printer_toner_name_db);
						make_alert_hd(1, $printer_trans['uniq'], $printer_toner_name_db);
						$email++;
					}
				}
				$ti++;
			} while (isset($TONERS[$ti]));
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t</consumables>";
			// Include the page count if available
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<pages value=\"total pages printed\">".@get_printer_pages_total($printers['ip'][$li], $printers['ro_comm'][$li],$printer_trans['id'])."</pages>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<errors>".$errors."</errors>";

			
		} else {
			// We have an offline device!
			// Get the little info we have and assume it's correct
			$printer_id = check_deivce_id($printers['ip'][$li]);
			// This should retrieve the last SNMP_Device Unique_ID that was detected. (See function for info)
			$printer_trans = get_device_info_offline($printer_id);
			// If we have device info display it, if not then display unknown.
			if(empty($printer_trans)) {
				$printer_trans['desc'] = "Unknown Device";
			}
			if(!empty($printer_id)) {
// Offline Device Error (5)
				make_alert(5, $printer_id);
				make_alert_hd(5, $printer_id);
				$email++;
			}
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<description>".$printer_trans['desc']."</description>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<status>Unable to Connect to Device</status>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<address>".$printers['ip'][$li]."</address>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<consumables></consumables>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<pages></pages>";
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t\t<errors>1</errors>";
		}
			$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t\t</printer>";
			
			// If the printer has no errors this run then clear any errors from the log as they must be fixed.
			if ($email == 0) {
				reset_alerts($printer_trans['uniq']);
				reset_alerts_hd($printer_trans['uniq']);
			}
			$li++;
		} while (isset($printers['id'][$li]));
		$PAGE_OUTPUT = $PAGE_OUTPUT."\n\t</status>";
	
// Cache me baby
		// Open the cache file for overwriting.
		$fopen = @fopen($PAGE_NAME , "w");
		$write = @fwrite($fopen, $PAGE_OUTPUT);
		@fclose($fopen);
		// Log any cache errors
		if(($write != FALSE)&&(isset($write))) {
			@logger("success","Cache","Printer Stats Cached.");
		} else {
			@logger("error","Cache","Error Caching Printer Stats");
		}
	} else {
		@logger("success","Cache","Cache update for Printer Stats Not needed");
	}
	
?>